##
# This code was generated by
# \ / _    _  _|   _  _
#  | (_)\/(_)(_|\/| |(/_  v1.0.0
#       /       /
# 
# frozen_string_literal: true

module Twilio
  module REST
    class Sync < Domain
      class V1 < Version
        class ServiceContext < InstanceContext
          ##
          # PLEASE NOTE that this class contains beta products that are subject to change. Use them with caution.
          class DocumentList < ListResource
            ##
            # Initialize the DocumentList
            # @param [Version] version Version that contains the resource
            # @param [String] service_sid The unique SID identifier of the Service Instance
            #   that hosts this Document.
            # @return [DocumentList] DocumentList
            def initialize(version, service_sid: nil)
              super(version)

              # Path Solution
              @solution = {service_sid: service_sid}
              @uri = "/Services/#{@solution[:service_sid]}/Documents"
            end

            ##
            # Retrieve a single page of DocumentInstance records from the API.
            # Request is executed immediately.
            # @param [String] unique_name Human-readable name for this document
            # @param [Hash] data JSON data to be stored in this document
            # @param [String] ttl Time-to-live of this Document in seconds, defaults to no
            #   expiration. In the range [1, 31 536 000 (1 year)], or 0 for infinity.
            # @return [DocumentInstance] Newly created DocumentInstance
            def create(unique_name: :unset, data: :unset, ttl: :unset)
              data = Twilio::Values.of({
                  'UniqueName' => unique_name,
                  'Data' => Twilio.serialize_object(data),
                  'Ttl' => ttl,
              })

              payload = @version.create(
                  'POST',
                  @uri,
                  data: data
              )

              DocumentInstance.new(@version, payload, service_sid: @solution[:service_sid], )
            end

            ##
            # Lists DocumentInstance records from the API as a list.
            # Unlike stream(), this operation is eager and will load `limit` records into
            # memory before returning.
            # @param [Integer] limit Upper limit for the number of records to return. stream()
            #    guarantees to never return more than limit.  Default is no limit
            # @param [Integer] page_size Number of records to fetch per request, when
            #    not set will use the default value of 50 records.  If no page_size is defined
            #    but a limit is defined, stream() will attempt to read the limit with the most
            #    efficient page size, i.e. min(limit, 1000)
            # @return [Array] Array of up to limit results
            def list(limit: nil, page_size: nil)
              self.stream(limit: limit, page_size: page_size).entries
            end

            ##
            # Streams DocumentInstance records from the API as an Enumerable.
            # This operation lazily loads records as efficiently as possible until the limit
            # is reached.
            # @param [Integer] limit Upper limit for the number of records to return. stream()
            #    guarantees to never return more than limit. Default is no limit.
            # @param [Integer] page_size Number of records to fetch per request, when
            #    not set will use the default value of 50 records. If no page_size is defined
            #    but a limit is defined, stream() will attempt to read the limit with the most
            #    efficient page size, i.e. min(limit, 1000)
            # @return [Enumerable] Enumerable that will yield up to limit results
            def stream(limit: nil, page_size: nil)
              limits = @version.read_limits(limit, page_size)

              page = self.page(page_size: limits[:page_size], )

              @version.stream(page, limit: limits[:limit], page_limit: limits[:page_limit])
            end

            ##
            # When passed a block, yields DocumentInstance records from the API.
            # This operation lazily loads records as efficiently as possible until the limit
            # is reached.
            def each
              limits = @version.read_limits

              page = self.page(page_size: limits[:page_size], )

              @version.stream(page,
                              limit: limits[:limit],
                              page_limit: limits[:page_limit]).each {|x| yield x}
            end

            ##
            # Retrieve a single page of DocumentInstance records from the API.
            # Request is executed immediately.
            # @param [String] page_token PageToken provided by the API
            # @param [Integer] page_number Page Number, this value is simply for client state
            # @param [Integer] page_size Number of records to return, defaults to 50
            # @return [Page] Page of DocumentInstance
            def page(page_token: :unset, page_number: :unset, page_size: :unset)
              params = Twilio::Values.of({
                  'PageToken' => page_token,
                  'Page' => page_number,
                  'PageSize' => page_size,
              })
              response = @version.page(
                  'GET',
                  @uri,
                  params
              )
              DocumentPage.new(@version, response, @solution)
            end

            ##
            # Retrieve a single page of DocumentInstance records from the API.
            # Request is executed immediately.
            # @param [String] target_url API-generated URL for the requested results page
            # @return [Page] Page of DocumentInstance
            def get_page(target_url)
              response = @version.domain.request(
                  'GET',
                  target_url
              )
              DocumentPage.new(@version, response, @solution)
            end

            ##
            # Provide a user friendly representation
            def to_s
              '#<Twilio.Sync.V1.DocumentList>'
            end
          end

          ##
          # PLEASE NOTE that this class contains beta products that are subject to change. Use them with caution.
          class DocumentPage < Page
            ##
            # Initialize the DocumentPage
            # @param [Version] version Version that contains the resource
            # @param [Response] response Response from the API
            # @param [Hash] solution Path solution for the resource
            # @return [DocumentPage] DocumentPage
            def initialize(version, response, solution)
              super(version, response)

              # Path Solution
              @solution = solution
            end

            ##
            # Build an instance of DocumentInstance
            # @param [Hash] payload Payload response from the API
            # @return [DocumentInstance] DocumentInstance
            def get_instance(payload)
              DocumentInstance.new(@version, payload, service_sid: @solution[:service_sid], )
            end

            ##
            # Provide a user friendly representation
            def to_s
              '<Twilio.Sync.V1.DocumentPage>'
            end
          end

          ##
          # PLEASE NOTE that this class contains beta products that are subject to change. Use them with caution.
          class DocumentContext < InstanceContext
            ##
            # Initialize the DocumentContext
            # @param [Version] version Version that contains the resource
            # @param [String] service_sid The service_sid
            # @param [String] sid The sid
            # @return [DocumentContext] DocumentContext
            def initialize(version, service_sid, sid)
              super(version)

              # Path Solution
              @solution = {service_sid: service_sid, sid: sid, }
              @uri = "/Services/#{@solution[:service_sid]}/Documents/#{@solution[:sid]}"

              # Dependents
              @document_permissions = nil
            end

            ##
            # Fetch a DocumentInstance
            # @return [DocumentInstance] Fetched DocumentInstance
            def fetch
              params = Twilio::Values.of({})

              payload = @version.fetch(
                  'GET',
                  @uri,
                  params,
              )

              DocumentInstance.new(@version, payload, service_sid: @solution[:service_sid], sid: @solution[:sid], )
            end

            ##
            # Deletes the DocumentInstance
            # @return [Boolean] true if delete succeeds, true otherwise
            def delete
              @version.delete('delete', @uri)
            end

            ##
            # Update the DocumentInstance
            # @param [Hash] data Contains an arbitrary JSON object to be stored in this
            #   Document. Serialized to string to respect HTTP form input, up to 16KB.
            # @param [String] ttl New time-to-live of this Document in seconds. In the range
            #   [1, 31 536 000 (1 year)], or 0 for infinity.
            # @return [DocumentInstance] Updated DocumentInstance
            def update(data: :unset, ttl: :unset)
              data = Twilio::Values.of({'Data' => Twilio.serialize_object(data), 'Ttl' => ttl, })

              payload = @version.update(
                  'POST',
                  @uri,
                  data: data,
              )

              DocumentInstance.new(@version, payload, service_sid: @solution[:service_sid], sid: @solution[:sid], )
            end

            ##
            # Access the document_permissions
            # @return [DocumentPermissionList]
            # @return [DocumentPermissionContext] if identity was passed.
            def document_permissions(identity=:unset)
              raise ArgumentError, 'identity cannot be nil' if identity.nil?

              if identity != :unset
                return DocumentPermissionContext.new(@version, @solution[:service_sid], @solution[:sid], identity, )
              end

              unless @document_permissions
                @document_permissions = DocumentPermissionList.new(
                    @version,
                    service_sid: @solution[:service_sid],
                    document_sid: @solution[:sid],
                )
              end

              @document_permissions
            end

            ##
            # Provide a user friendly representation
            def to_s
              context = @solution.map {|k, v| "#{k}: #{v}"}.join(',')
              "#<Twilio.Sync.V1.DocumentContext #{context}>"
            end
          end

          ##
          # PLEASE NOTE that this class contains beta products that are subject to change. Use them with caution.
          class DocumentInstance < InstanceResource
            ##
            # Initialize the DocumentInstance
            # @param [Version] version Version that contains the resource
            # @param [Hash] payload payload that contains response from Twilio
            # @param [String] service_sid The unique SID identifier of the Service Instance
            #   that hosts this Document.
            # @param [String] sid The sid
            # @return [DocumentInstance] DocumentInstance
            def initialize(version, payload, service_sid: nil, sid: nil)
              super(version)

              # Marshaled Properties
              @properties = {
                  'sid' => payload['sid'],
                  'unique_name' => payload['unique_name'],
                  'account_sid' => payload['account_sid'],
                  'service_sid' => payload['service_sid'],
                  'url' => payload['url'],
                  'links' => payload['links'],
                  'revision' => payload['revision'],
                  'data' => payload['data'],
                  'date_expires' => Twilio.deserialize_iso8601_datetime(payload['date_expires']),
                  'date_created' => Twilio.deserialize_iso8601_datetime(payload['date_created']),
                  'date_updated' => Twilio.deserialize_iso8601_datetime(payload['date_updated']),
                  'created_by' => payload['created_by'],
              }

              # Context
              @instance_context = nil
              @params = {'service_sid' => service_sid, 'sid' => sid || @properties['sid'], }
            end

            ##
            # Generate an instance context for the instance, the context is capable of
            # performing various actions.  All instance actions are proxied to the context
            # @return [DocumentContext] DocumentContext for this DocumentInstance
            def context
              unless @instance_context
                @instance_context = DocumentContext.new(@version, @params['service_sid'], @params['sid'], )
              end
              @instance_context
            end

            ##
            # @return [String] The unique 34-character SID identifier of the Document.
            def sid
              @properties['sid']
            end

            ##
            # @return [String] The unique and addressable name of this Document.
            def unique_name
              @properties['unique_name']
            end

            ##
            # @return [String] The unique SID identifier of the Twilio Account.
            def account_sid
              @properties['account_sid']
            end

            ##
            # @return [String] The unique SID identifier of the Service Instance that hosts this Document.
            def service_sid
              @properties['service_sid']
            end

            ##
            # @return [String] The absolute URL for this Document.
            def url
              @properties['url']
            end

            ##
            # @return [String] A dictionary of URL links to nested resources of this Document.
            def links
              @properties['links']
            end

            ##
            # @return [String] Contains the current revision of this Document, represented by a string identifier.
            def revision
              @properties['revision']
            end

            ##
            # @return [Hash] Contains arbitrary user-defined, schema-less data that this Document stores, represented by a JSON object, up to 16KB.
            def data
              @properties['data']
            end

            ##
            # @return [Time] Contains the date this Document expires and gets deleted automatically.
            def date_expires
              @properties['date_expires']
            end

            ##
            # @return [Time] The date this Document was created, given in UTC ISO 8601 format.
            def date_created
              @properties['date_created']
            end

            ##
            # @return [Time] Specifies the date this Document was last updated, given in UTC ISO 8601 format.
            def date_updated
              @properties['date_updated']
            end

            ##
            # @return [String] The identity of the Document creator.
            def created_by
              @properties['created_by']
            end

            ##
            # Fetch a DocumentInstance
            # @return [DocumentInstance] Fetched DocumentInstance
            def fetch
              context.fetch
            end

            ##
            # Deletes the DocumentInstance
            # @return [Boolean] true if delete succeeds, true otherwise
            def delete
              context.delete
            end

            ##
            # Update the DocumentInstance
            # @param [Hash] data Contains an arbitrary JSON object to be stored in this
            #   Document. Serialized to string to respect HTTP form input, up to 16KB.
            # @param [String] ttl New time-to-live of this Document in seconds. In the range
            #   [1, 31 536 000 (1 year)], or 0 for infinity.
            # @return [DocumentInstance] Updated DocumentInstance
            def update(data: :unset, ttl: :unset)
              context.update(data: data, ttl: ttl, )
            end

            ##
            # Access the document_permissions
            # @return [document_permissions] document_permissions
            def document_permissions
              context.document_permissions
            end

            ##
            # Provide a user friendly representation
            def to_s
              values = @params.map{|k, v| "#{k}: #{v}"}.join(" ")
              "<Twilio.Sync.V1.DocumentInstance #{values}>"
            end

            ##
            # Provide a detailed, user friendly representation
            def inspect
              values = @properties.map{|k, v| "#{k}: #{v}"}.join(" ")
              "<Twilio.Sync.V1.DocumentInstance #{values}>"
            end
          end
        end
      end
    end
  end
end